from typing import Optional
from pydantic import Field

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle, strEnum
from gso.products.product_blocks.device \
    import DeviceBlock, DeviceBlockInactive, DeviceBlockProvisioning
import ipaddress


class IptrunkType(strEnum):
    Dark_fiber = "Dark_fiber"
    Leased = "Leased"


class IptrunkBlockInactive(ProductBlockModel,
                           lifecycle=[SubscriptionLifecycle.INITIAL],
                           product_block_name="IptrunkBlock"):
    geant_s_sid: Optional[str] = None
    iptrunk_description: Optional[str] = None
    iptrunk_type: Optional[IptrunkType] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sideA_node: DeviceBlockInactive
    iptrunk_sideA_ae_iface: Optional[str] = None
    iptrunk_sideA_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideA_ae_members: Optional[list] = None
    iptrunk_sideA_ae_members_description: Optional[list] = None
    #
    iptrunk_sideB_node: DeviceBlockInactive
    iptrunk_sideB_ae_iface: Optional[str] = None
    iptrunk_sideB_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideB_ae_members: Optional[list] = None
    iptrunk_sideB_ae_members_description: Optional[list] = None


class IptrunkBlockProvisioning(IptrunkBlockInactive,
                               lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    geant_s_sid: Optional[str] = None
    iptrunk_description: Optional[str] = None
    iptrunk_type: Optional[IptrunkType] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sideA_node: DeviceBlockProvisioning
    iptrunk_sideA_ae_iface: Optional[str] = None
    iptrunk_sideA_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideA_ae_members: list[str] = Field(default_factory=list)
    iptrunk_sideA_ae_members_description: list[str] = Field(default_factory=list)
    #
    iptrunk_sideB_node: DeviceBlockProvisioning
    iptrunk_sideB_ae_iface: Optional[str] = None
    iptrunk_sideB_ae_geant_a_sid: Optional[str] = None
    iptrunk_sideB_ae_members: list[str] = Field(default_factory=list)
    iptrunk_sideB_ae_members_description: list[str] = Field(default_factory=list)


class IptrunkBlock(IptrunkBlockProvisioning,
                   lifecycle=[SubscriptionLifecycle.ACTIVE]):
    geant_s_sid: str
    iptrunk_description: str
    iptrunk_type: IptrunkType
    iptrunk_speed: str
    iptrunk_minimum_links: int
    iptrunk_ipv4_network: ipaddress.IPv4Network
    iptrunk_ipv6_network: ipaddress.IPv6Network
    #
    iptrunk_sideA_node: DeviceBlock
    iptrunk_sideA_ae_iface: str
    iptrunk_sideA_ae_geant_a_sid: str
    iptrunk_sideA_ae_members: list[str] = Field(default_factory=list)
    iptrunk_sideA_ae_members_description: list[str] = Field(default_factory=list)
    #
    iptrunk_sideB_node: DeviceBlock
    iptrunk_sideB_ae_iface: str
    iptrunk_sideB_ae_geant_a_sid: str
    iptrunk_sideB_ae_members: list[str] = Field(default_factory=list)
    iptrunk_sideB_ae_members_description: list[str] = Field(default_factory=list)
